//
// PS2PC: An Intel x86 emulator for MIPS R5900 (Sony PS2).
// Coded by Mike Tedder / breakpoint - bpoint@parodius.com
// http://ps2pc.parodius.com
//
// This project is released as 'open source'.  You may make any modifications
// or adjustments to the code and submit a patch to be applied to the project.
// See the web page for more details.
//
//-----------------------------------------------------------------------

#include "types.h"
#include "x86.h"
#include "x86mov.h"
#include "r59basic.h"


//=====================
//        MOV
//=====================

int x86MOV_EbGb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86MOV_EvGv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86MOV_GbEb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86MOV_GvEv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86MOV_EwSw(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86MOV_SwEw(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86MOV_ALOb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86MOV_eAXOv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86MOV_ObAL(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86MOV_OveAX(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86MOV_ALIb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	uchar val;

	// Get immediate byte value.
	val = *(in + 1);

	// Encode it...
	* out      = r5900ADDIU(rS0, rR0, 0xFF00);				// S0   = 0xFFFFFF00
	*(out + 1) = r5900AND  (rS1, rT0, rS0);					// EAX &= 0xFFFFFF00
	*(out + 2) = r5900ORI  (rT0, rS1, val);					// EAX |= val

	*skip += 3;
	return 2;
}


int x86MOV_CLIb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	uchar val;

	// Get immediate byte value.
	val = *(in + 1);

	// Encode it...
	* out      = r5900ADDIU(rS0, rR0, 0xFF00);				// S0   = 0xFFFFFF00
	*(out + 1) = r5900AND  (rS1, rT2, rS0);					// ECX &= 0xFFFFFF00
	*(out + 2) = r5900ORI  (rT2, rS1, val);					// ECX |= val

	*skip += 3;
	return 2;
}


int x86MOV_DLIb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	uchar val;

	// Get immediate byte value.
	val = *(in + 1);

	// Encode it...
	* out      = r5900ADDIU(rS0, rR0, 0xFF00);				// S0   = 0xFFFFFF00
	*(out + 1) = r5900AND  (rS1, rT3, rS0);					// EDX &= 0xFFFFFF00
	*(out + 2) = r5900ORI  (rT3, rS1, val);					// EDX |= val

	*skip += 3;
	return 2;
}


int x86MOV_BLIb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	uchar val;

	// Get immediate byte value.
	val = *(in + 1);

	// Encode it...
	* out      = r5900ADDIU(rS0, rR0, 0xFF00);				// S0   = 0xFFFFFF00
	*(out + 1) = r5900AND  (rS1, rT1, rS0);					// EBX &= 0xFFFFFF00
	*(out + 2) = r5900ORI  (rT1, rS1, val);					// EBX |= val

	*skip += 3;
	return 2;
}


int x86MOV_AHIb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	uchar val;

	// Get immediate byte value.
	val = *(in + 1);

	// Encode it...
	* out      = r5900LUI(rS0, 0xFFFF);						// S0   = 0xFFFF0000
	*(out + 1) = r5900ORI(rS1, rS0, 0x00FF);				// S1   = 0xFFFF00FF
	*(out + 2) = r5900AND(rS0, rT0, rS1);					// EAX &= 0xFFFF00FF
	*(out + 3) = r5900ORI(rT0, rS0, val << 8);				// EAX |= val << 8

	*skip += 4;
	return 2;
}


int x86MOV_CHIb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	uchar val;

	// Get immediate byte value.
	val = *(in + 1);

	// Encode it...
	* out      = r5900LUI(rS0, 0xFFFF);						// S0   = 0xFFFF0000
	*(out + 1) = r5900ORI(rS1, rS0, 0x00FF);				// S1   = 0xFFFF00FF
	*(out + 2) = r5900AND(rS0, rT2, rS1);					// ECX &= 0xFFFF00FF
	*(out + 3) = r5900ORI(rT2, rS0, val << 8);				// ECX |= val << 8

	*skip += 4;
	return 2;
}


int x86MOV_DHIb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	uchar val;

	// Get immediate byte value.
	val = *(in + 1);

	// Encode it...
	* out      = r5900LUI(rS0, 0xFFFF);						// S0   = 0xFFFF0000
	*(out + 1) = r5900ORI(rS1, rS0, 0x00FF);				// S1   = 0xFFFF00FF
	*(out + 2) = r5900AND(rS0, rT3, rS1);					// EDX &= 0xFFFF00FF
	*(out + 3) = r5900ORI(rT3, rS0, val << 8);				// EDX |= val << 8

	*skip += 4;
	return 2;
}


int x86MOV_BHIb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	uchar val;

	// Get immediate byte value.
	val = *(in + 1);

	// Encode it...
	* out      = r5900LUI(rS0, 0xFFFF);						// S0   = 0xFFFF0000
	*(out + 1) = r5900ORI(rS1, rS0, 0x00FF);				// S1   = 0xFFFF00FF
	*(out + 2) = r5900AND(rS0, rT1, rS1);					// EBX &= 0xFFFF00FF
	*(out + 3) = r5900ORI(rT1, rS0, val << 8);				// EBX |= val << 8

	*skip += 4;
	return 2;
}


int x86MOV_eAXIv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	ushort sval;

	if (in32BitMode)
	{
		// Get upper immediate word value.
		sval = MAKE_WORD(*(in + 4), *(in + 3));

		// Encode it...
		*out = r5900LUI(rT0, sval);							// EAX = 0xuuuu0000

		// Get lower immediate word value.
		sval = MAKE_WORD(*(in + 2), *(in + 1));

		// And encode that...
		*(out + 1) = r5900ORI(rT0, rT0, sval);				// EAX = 0xuuuullll

		*skip += 2;
		return 5;
	}
	else
	{
		// Get immediate word value.
		sval = MAKE_WORD(*(in + 2), *(in + 1));

		// Encode it...
		* out      = r5900LUI(rS0, 0xFFFF);					// S0   = 0xFFFF0000
		*(out + 1) = r5900AND(rS1, rT0, rS0);				// EAX &= 0xFFFF0000
		*(out + 2) = r5900ORI(rT0, rS1, sval);				// EAX |= 0x0000llll

		*skip += 3;
		return 3;
	}
}


int x86MOV_eCXIv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	ushort sval;

	if (in32BitMode)
	{
		// Get upper immediate word value.
		sval = MAKE_WORD(*(in + 4), *(in + 3));

		// Encode it...
		*out = r5900LUI(rT2, sval);							// ECX = 0xuuuu0000

		// Get lower immediate word value.
		sval = MAKE_WORD(*(in + 2), *(in + 1));

		// And encode that...
		*(out + 1) = r5900ORI(rT2, rT2, sval);				// ECX = 0xuuuullll

		*skip += 2;
		return 5;
	}
	else
	{
		// Get immediate word value.
		sval = MAKE_WORD(*(in + 2), *(in + 1));

		// Encode it...
		* out      = r5900LUI(rS0, 0xFFFF);					// S0   = 0xFFFF0000
		*(out + 1) = r5900AND(rS1, rT2, rS0);				// ECX &= 0xFFFF0000
		*(out + 2) = r5900ORI(rT2, rS1, sval);				// ECX |= 0x0000llll

		*skip += 3;
		return 3;
	}
}


int x86MOV_eDXIv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	ushort sval;

	if (in32BitMode)
	{
		// Get upper immediate word value.
		sval = MAKE_WORD(*(in + 4), *(in + 3));

		// Encode it...
		*out = r5900LUI(rT3, sval);							// EDX = 0xuuuu0000

		// Get lower immediate word value.
		sval = MAKE_WORD(*(in + 2), *(in + 1));

		// And encode that...
		*(out + 1) = r5900ORI(rT3, rT3, sval);				// EDX = 0xuuuullll

		*skip += 2;
		return 5;
	}
	else
	{
		// Get immediate word value.
		sval = MAKE_WORD(*(in + 2), *(in + 1));

		// Encode it...
		* out      = r5900LUI(rS0, 0xFFFF);					// S0   = 0xFFFF0000
		*(out + 1) = r5900AND(rS1, rT3, rS0);				// EDX &= 0xFFFF0000
		*(out + 2) = r5900ORI(rT3, rS1, sval);				// EDX |= 0x0000llll

		*skip += 3;
		return 3;
	}
}


int x86MOV_eBXIv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	ushort sval;

	if (in32BitMode)
	{
		// Get upper immediate word value.
		sval = MAKE_WORD(*(in + 4), *(in + 3));

		// Encode it...
		*out = r5900LUI(rT1, sval);							// EBX = 0xuuuu0000

		// Get lower immediate word value.
		sval = MAKE_WORD(*(in + 2), *(in + 1));

		// And encode that...
		*(out + 1) = r5900ORI(rT1, rT1, sval);				// EBX = 0xuuuullll

		*skip += 2;
		return 5;
	}
	else
	{
		// Get immediate word value.
		sval = MAKE_WORD(*(in + 2), *(in + 1));

		// Encode it...
		* out      = r5900LUI(rS0, 0xFFFF);					// S0   = 0xFFFF0000
		*(out + 1) = r5900AND(rS1, rT1, rS0);				// EBX &= 0xFFFF0000
		*(out + 2) = r5900ORI(rT1, rS1, sval);				// EBX |= 0x0000llll

		*skip += 3;
		return 3;
	}
}


int x86MOV_eSPIv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	ushort sval;

	if (in32BitMode)
	{
		// Get upper immediate word value.
		sval = MAKE_WORD(*(in + 4), *(in + 3));

		// Encode it...
		*out = r5900LUI(rT7, sval);							// ESP = 0xuuuu0000

		// Get lower immediate word value.
		sval = MAKE_WORD(*(in + 2), *(in + 1));

		// And encode that...
		*(out + 1) = r5900ORI(rT7, rT7, sval);				// ESP = 0xuuuullll

		*skip += 2;
		return 5;
	}
	else
	{
		// Get immediate word value.
		sval = MAKE_WORD(*(in + 2), *(in + 1));

		// Encode it...
		* out      = r5900LUI(rS0, 0xFFFF);					// S0   = 0xFFFF0000
		*(out + 1) = r5900AND(rS1, rT7, rS0);				// ESP &= 0xFFFF0000
		*(out + 2) = r5900ORI(rT7, rS1, sval);				// ESP |= 0x0000llll

		*skip += 3;
		return 3;
	}
}


int x86MOV_eBPIv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	ushort sval;

	if (in32BitMode)
	{
		// Get upper immediate word value.
		sval = MAKE_WORD(*(in + 4), *(in + 3));

		// Encode it...
		*out = r5900LUI(rT6, sval);							// EBP = 0xuuuu0000

		// Get lower immediate word value.
		sval = MAKE_WORD(*(in + 2), *(in + 1));

		// And encode that...
		*(out + 1) = r5900ORI(rT6, rT6, sval);				// EBP = 0xuuuullll

		*skip += 2;
		return 5;
	}
	else
	{
		// Get immediate word value.
		sval = MAKE_WORD(*(in + 2), *(in + 1));

		// Encode it...
		* out      = r5900LUI(rS0, 0xFFFF);					// S0   = 0xFFFF0000
		*(out + 1) = r5900AND(rS1, rT6, rS0);				// EBP &= 0xFFFF0000
		*(out + 2) = r5900ORI(rT6, rS1, sval);				// EBP |= 0x0000llll

		*skip += 3;
		return 3;
	}
}


int x86MOV_eSIIv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	ushort sval;

	if (in32BitMode)
	{
		// Get upper immediate word value.
		sval = MAKE_WORD(*(in + 4), *(in + 3));

		// Encode it...
		*out = r5900LUI(rT4, sval);							// ESI = 0xuuuu0000

		// Get lower immediate word value.
		sval = MAKE_WORD(*(in + 2), *(in + 1));

		// And encode that...
		*(out + 1) = r5900ORI(rT4, rT4, sval);				// ESI = 0xuuuullll

		*skip += 2;
		return 5;
	}
	else
	{
		// Get immediate word value.
		sval = MAKE_WORD(*(in + 2), *(in + 1));

		// Encode it...
		* out      = r5900LUI(rS0, 0xFFFF);					// S0   = 0xFFFF0000
		*(out + 1) = r5900AND(rS1, rT4, rS0);				// ESI &= 0xFFFF0000
		*(out + 2) = r5900ORI(rT4, rS1, sval);				// ESI |= 0x0000llll

		*skip += 3;
		return 3;
	}
}


int x86MOV_eDIIv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	ushort sval;

	if (in32BitMode)
	{
		// Get upper immediate word value.
		sval = MAKE_WORD(*(in + 4), *(in + 3));

		// Encode it...
		*out = r5900LUI(rT5, sval);							// EDI = 0xuuuu0000

		// Get lower immediate word value.
		sval = MAKE_WORD(*(in + 2), *(in + 1));

		// And encode that...
		*(out + 1) = r5900ORI(rT5, rT5, sval);				// EDI = 0xuuuullll

		*skip += 2;
		return 5;
	}
	else
	{
		// Get immediate word value.
		sval = MAKE_WORD(*(in + 2), *(in + 1));

		// Encode it...
		* out      = r5900LUI(rS0, 0xFFFF);					// S0   = 0xFFFF0000
		*(out + 1) = r5900AND(rS1, rT5, rS0);				// EDI &= 0xFFFF0000
		*(out + 2) = r5900ORI(rT5, rS1, sval);				// EDI |= 0x0000llll

		*skip += 3;
		return 3;
	}
}


int x86MOV_EbIb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86MOV_EvIv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


